/*
 * atmelio.cc
 * 
 * Atmel programmer: Atmel chip IO virtual base class routines 
 * using latch16bit for communication. 
 * 
 * Copyright (c) 2003--2005 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#include "atmelio.h"

#include <stdlib.h>
#include <assert.h>


void *AtmelIO::operator new(size_t size)
{
	void *ptr=malloc(size);
	assert(ptr);
	return(ptr);
}

void AtmelIO::operator delete(void *ptr)
{
	if(ptr)  free(ptr);
}


ssize_t AtmelIO::ReadFlash(size_t /*addr*/,size_t /*size*/,u_int16_t * /*buf*/)
{
	// Must be overridden if used. 
	assert(0);
	return(-1);
}


int AtmelIO::WriteFlash(size_t /*addr*/,size_t /*size*/,const u_int16_t * /*buf*/)
{
	// Must be overridden if used. 
	assert(0);
	return(-1);
}


ssize_t AtmelIO::ReadEEPROM(size_t /*addr*/,size_t /*size*/,uchar * /*buf*/)
{
	// Must be overridden if used. 
	assert(0);
	return(-1);
}


int AtmelIO::WriteEEPROM(size_t /*addr*/,size_t /*size*/,const uchar * /*buf*/)
{
	// Must be overridden if used. 
	assert(0);
	return(-1);
}


int AtmelIO::ChipErase()
{
	// Must be overridden if used. 
	assert(0);
	return(-1);
}


int AtmelIO::ReadFuseAndLock(uchar * /*buf*/)
{
	// Must be overridden if used. 
	assert(0);
	return(-1);
}


AtmelIO::ATDevice AtmelIO::ReadSignature(Signature * /*_sig*/)
{
	// Must be overridden if used. 
	assert(0);
	return(AD_Unknown);
}


void AtmelIO::WriteFuse(uchar /*val*/,char /*is_high*/)
{
	// Must be overridden if used. 
	assert(0);
}


void AtmelIO::EnterProgMode(volatile int * /*caught_signal*/)
{
	// Must be overridden if used. 
	assert(0);
}


void AtmelIO::LeaveProgMode(volatile int * /*caught_signal*/)
{
	// Must be overridden if used. 
	assert(0);
}


int AtmelIO::Connect()
{
	// Must be overridden if used. 
	assert(0);
	return(-1);
}

int AtmelIO::Disconnect()
{
	// Must be overridden if used. 
	assert(0);
	return(-1);
}


AtmelIO::AtmelIO(LPPortIO *pio,uchar strobe_line,uchar ctlsel_line) : 
	l16b(pio),
	config()
{
	l16b.Init(strobe_line,ctlsel_line);
}

AtmelIO::~AtmelIO()
{
}


/******************************************************************************/


void AtmelIO::Signature::ParseSignature(ATConfig *config)
{
	vendor="[unknown]";
	flash_size=0;
	device="[unknown]";
	config->device=AD_Unknown;
	
	switch(_vendor)
	{
		case 0x1e:  vendor="Atmel";  break;
	}
	
	switch(_flash_size)
	{
		case 0x90:
			flash_size=512;
			
			break;
		case 0x91:
			flash_size=1024;
			
			if(_device==0x01)
			{
				device="AT90S2313";
				eeprom_size=128;
				config->device=AD_AT90S2313;
				config->page_size=0;
				config->fuse_lock_bytes=1;
				config->has_erase_busy=0;
			}
			
			break;
		case 0x92:
			flash_size=2048;
			
			if(_device==0x03)
			{
				device="AT90S4433";
				eeprom_size=256;
				config->device=AD_AT90S4433;
				config->page_size=0;
				config->fuse_lock_bytes=2;
				config->has_erase_busy=0;
			}
			
			break;
		case 0x93:
			flash_size=4096;
			
			if(_device==0x06)
			{
				device="ATMega8515";
				eeprom_size=512;
				config->device=AD_ATMega8515;
				config->page_size=64;
				config->fuse_lock_bytes=3;
				config->has_erase_busy=1;
			}
			
			break;
		case 0x94:
			flash_size=8192;
			
			if(_device==0x01)
			{
				device="ATMega161";
				eeprom_size=512;
				config->device=AD_ATMega161;
				config->page_size=128;
				config->fuse_lock_bytes=2;
				config->has_erase_busy=1;
			}
			
			break;
	}
}


AtmelIO::Signature::Signature()
{
	vendor=NULL;
	flash_size=0;   // in WORDS
	eeprom_size=0;  // in BYTES
	device=NULL;
	_vendor=0xff;
	_flash_size=0xff;
	_device=0xff;
}

AtmelIO::ATConfig::ATConfig() : 
	sig()
{
	device=AD_Unknown;
	page_size=0;
	fuse_lock_bytes=1;
	has_erase_busy=0;
}
