/*
 * atmelio.h
 * 
 * Atmel programmer: Atmel chip IO routines using latch16bit for 
 * communication. 
 * Virtual base class for serial (SPI) and parallel transfer methods. 
 * 
 * Copyright (c) 2003--2005 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _ATMELPRG_ATMELIO_H_
#define _ATMELPRG_ATMELIO_H_

#include "l16bitio.h"


class AtmelIO
{
	public:
		struct ATConfig;
		
		struct Signature
		{
			// Parsed data: 
			const char *vendor;
			size_t flash_size;   // in WORDS
			size_t eeprom_size;  // in BYTES
			const char *device;
			// Raw data: 
			uchar _vendor;
			uchar _flash_size;
			uchar _device;
			
			Signature();
			~Signature() {}
			
			// After having set _vendor,_flash_size,_device, use this 
			// routine to "parse" these and set up vendor,..,device. 
			// Pass config pointer to have it updated. 
			void ParseSignature(ATConfig *config);
		};
		
		enum ATDevice
		{
			AD_Unknown=0,
			AD_AT90S2313,
			AD_AT90S4433,
			AD_ATMega161,
			AD_ATMega8515,
		};
		
		struct ATConfig
		{
			// The signature of the device: 
			Signature sig;
			// The device identifier (numeric value of sig.device): 
			ATDevice device;
			// This specifies if the chip has a "page load" pin (!=0) or 
			// not (0)and specifies the page size in bytes. 
			// Refer to the parallel programming section of the data 
			// sheet for more info. 
			// For example, ATMega161 has a PAGEL pin, AT90S2313 not. 
			u_int16_t page_size;
			// Number of bytes for the fuse/lock bits. Normally 1 or 2. 
			// For example, ATMega161 has 2, AT90S2313 has 1, ATMega8515 has 3. 
			uchar fuse_lock_bytes;
			// If ChipErase command completion can be detected by 
			// watching RDY/BSY pin: 
			// (AT90S2313: no, ATMega161: yes)
			uchar has_erase_busy;
			
			ATConfig();
			~ATConfig() {}
		};
		
		void *operator new(size_t size);
		void operator delete(void *ptr);
		
	protected:
		// Latch16Bit board is used for communication: 
		Latch16BitIO l16b;
		
		// Some info about the chip we're connected to: 
		ATConfig config;
		
	public:
		AtmelIO(LPPortIO *pio,uchar strobe_line,uchar crlsel_line);
		virtual ~AtmelIO();
		
		// Be careful with this one: 
		Latch16BitIO *GetLatchIO()
			{  return(&l16b);  }
		
		// Do not modify this one: 
		const ATConfig *GetConfig() const
			{  return(&config);  }
		
		// You must call Connect() before you may call any of 
		// the IO functions. Connect() will switch on the output 
		// latches on the latch16bit board and make the input 
		// latches transparent. 
		// Use Disconnect() for the opposite behaviour. 
		// Return value:
		//   0 -> OK
		virtual int Connect();
		virtual int Disconnect();
		
		// Wait for entering/leaving programming mode. 
		// This may involve to wait until the user pressess some key. 
		// Use passed pointer to check for signals. 
		virtual void EnterProgMode(volatile int *caught_signal);
		virtual void LeaveProgMode(volatile int *caught_signal);
		
		// Read the signature bytes from the chip. 
		// *** Do this before doing anything else. ***
		// This sets internal flags (see ATConfig), e.g. if the device 
		// has a page load (PAGEL) pin or not. Knowing that is vital for 
		// Flash programming. 
		// If you are not interested in the signature, pass NULL. 
		// Return value: Device identifer or 
		//    AD_Unknown -> failure (unknown device or no device connected?)
		virtual ATDevice ReadSignature(Signature *sig=NULL);
		
		// Read fuse and lock bits and return them. 
		// fuse_and_lock must be capable of holding at least 2 bytes. 
		// Return value: 
		//   number of bytes in fuse_and_lock, 1 or 2, 0 if not possibe. 
		virtual int ReadFuseAndLock(uchar *fuse_and_lock);
		
		// Erase chip. 
		// return value: 
		//   0 (normally)
		//   1 must leave (and possibly re-enter) programming mode now. 
		virtual int ChipErase();
		
		// Read the flash memory: 
		//  addr: number of 2byte word to start reading at. 
		//  size: number of 2byte words to be read. 
		// Return value: size
		virtual ssize_t ReadFlash(size_t addr,size_t size,u_int16_t *buf);
		// Return value: 0
		virtual int WriteFlash(size_t addr,size_t size,const u_int16_t *buf);
		
		// Read the EEPROM memory: 
		//  addr: number of byte to start reading at. 
		//  size: number of bytes to be read. 
		// Return value: size
		virtual ssize_t ReadEEPROM(size_t addr,size_t size,uchar *buf);
		// Return value: 0
		virtual int WriteEEPROM(size_t addr,size_t size,const uchar *buf);
		
		// Write fuse byte (high byte if is_high is set). 
		virtual void WriteFuse(uchar val,char is_high);
};

#endif  /* _ATMELPRG_ATMELIO_H_ */
