/*
 * atmelio_p.h
 * 
 * Atmel programmer: Atmel chip IO routines using latch16bit for 
 * communication: Parallel transfer method. 
 * 
 * Copyright (c) 2003--2004 by Wolfgang Wieser (wwieser@gmx.de) 
 * 
 * This file may be distributed and/or modified under the terms of the 
 * GNU General Public License version 2 as published by the Free Software 
 * Foundation. 
 * 
 * This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 * WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * 
 */

#ifndef _ATMELPRG_ATMELIO_PARALLEL_H_
#define _ATMELPRG_ATMELIO_PARALLEL_H_

#include "atmelio.h"


class AtmelIO_Parallel : public AtmelIO
{
	public:
		// ATMEL commands: 
		enum
		{
			AT_CMD_NoOp=          0x00,   /* 0000 0000 */
			AT_CMD_ChipErase=     0x80,   /* 1000 0000 */
			AT_CMD_WriteFuse=     0x40,   /* 0100 0000 */
			AT_CMD_WriteLock=     0x20,   /* 0010 0000 */
			AT_CMD_WriteFlash=    0x10,   /* 0001 0000 */
			AT_CMD_WriteEEPROM=   0x11,   /* 0001 0001 */
			AT_CMD_ReadSignature= 0x08,   /* 0000 1000 */
			AT_CMD_ReadFuseLock=  0x04,   /* 0000 0100 */
			AT_CMD_ReadFlash=     0x02,   /* 0000 0010 */
			AT_CMD_ReadEEPROM=    0x03    /* 0000 0011 */
		};
		// ATMEL pins (output; "control")
		enum
		{
			AT_C_XTAL= 0x01,
			AT_C_OE=   0x02,  /* INV */
			AT_C_WR=   0x04,  /* INV */
			AT_C_BS=   0x08,
			AT_C_XA0=  0x10,
			AT_C_XA1=  0x20,
			AT_C_PAGEL=0x40
		};
		// ATMEL and programming board pins (input; "status")
		enum
		{
			AT_S_RDY=    0x01,
			AT_S_PWR=    0x02,   // power applied
			AT_S_PrgMode=0x08    // programming mode
		};
		
	private:
		// Latch16Bit board is used for communication: 
		//Latch16BitIO l16b;  <-- See base class. 
		
		// This is the current value on the control output 
		// (normally channel B of the latch16bit board): 
		uchar control_bits;
		
		// Some config: 
		const int DataChannel;
		const int ControlChannel;
		const int StatusChannel;
		
		// Some info about the chip we're connected to: 
		//ATConfig config;  <-- See base class. 
		
		// Change the value on the control output: 
		inline void _ATChgControl(uchar set,uchar clr)
		{
			control_bits|=set;
			control_bits&=~clr;
			l16b.WriteSingleByte(ControlChannel,control_bits);
		}
		
		// Make a short positive XTAL pulse: 
		inline void _ATPulseXTAL()
		{
			_ATChgControl(AT_C_XTAL,0x00);
			_ATChgControl(0x00,AT_C_XTAL);
		}
		
		// Make a short negative WR pulse: 
		inline void _ATPulseWR()
		{
			_ATChgControl(0x00,AT_C_WR);
			_ATChgControl(AT_C_WR,0x00);
		}
		
		// Make a short positive PAGEL pulse: 
		inline void _ATPulsePAGEL()
		{
			_ATChgControl(AT_C_PAGEL,0x00);
			_ATChgControl(0x00,AT_C_PAGEL);
		}
		
		// Load a command for the chip: 
		// cmd: one of the AT_CMD_* (see above) 
		void _ATLoadCommand(uchar cmd);
		// This is the opposite of loading a command: set the 
		// device idle: 
		void _ATSetIdle();
		// Load low/high part of data/address: 
		void _ATLoadDataLow(uchar data);
		void _ATLoadDataHigh(uchar data);
		void _ATLoadAddressLow(uchar ladr);
		void _ATLoadAddressHigh(uchar hadr);
		
		// Little IO wrapper: 
		inline void _ATDataOutb(uchar x)
		{  l16b.WriteSingleByte(DataChannel,x);  }
		
		// Read data byte/word explicitly enabling "output enable" on 
		// Atmel chip and disabling latch16bit output. 
		uchar _ATReadDataOE();
		u_int16_t _ATReadWordOE();
		
		void _ATWaitBusy();
		// Do not mix up with _ATLoadDataLow/High(). 
		void _ATWriteDataLow(uchar data);
		void _ATWriteDataHigh(uchar data);
		void _ATWriteWord(u_int16_t data);
		void _ATLatchWord(u_int16_t data);  // needs PAGEL
		
	public:
		AtmelIO_Parallel(LPPortIO *pio,uchar strobe_line,uchar crlsel_line);
		~AtmelIO_Parallel();
		
		// [overriding virtuals]
		int Connect();
		int Disconnect();
		void EnterProgMode(volatile int *caught_signal);
		void LeaveProgMode(volatile int *caught_signal);
		ATDevice ReadSignature(Signature *sig=NULL);
		int ReadFuseAndLock(uchar *fuse_and_lock);
		int ChipErase();
		ssize_t ReadFlash(size_t addr,size_t size,u_int16_t *buf);
		int WriteFlash(size_t addr,size_t size,const u_int16_t *buf);
		ssize_t ReadEEPROM(size_t addr,size_t size,uchar *buf);
		int WriteEEPROM(size_t addr,size_t size,const uchar *buf);
};

#endif  /* _ATMELPRG_ATMELIO_PARALLEL_H_ */
